/* filereq.cc
 * This file belongs to Worker, a filemanager for UNIX/X11.
 * Copyright (C) 2001 Ralf Hoffmann.
 * You can contact me at: ralf.hoffmann@epost.de
 *   or http://www.boomerangsworld.de/worker
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* $Id: filereq.cc,v 1.6 2001/11/03 17:10:04 ralf Exp $ */

#include "filereq.h"
#include <aguix/aguix.h>
#include <aguix/awindow.h>
#include <aguix/button.h>
#include <aguix/text.h>
#include <aguix/message.h>
#include <aguix/util.h>
#include <aguix/listview.h>
#include <aguix/stringgadget.h>

FileRequester::FileRequester(AGUIX *parent)
{
  aguix=parent;
  selfes=NULL;
  actdir=NULL;
  verz=NULL;
}

FileRequester::~FileRequester()
{
  reset();
  if(actdir!=NULL) _freesafe(actdir);
  if(verz!=NULL) delete verz;
}

int FileRequester::request(char *title,
                           char *dir,
                           char *oktext,
                           char *canceltext,
                           char *infotext)
{
  if((title==NULL)||(dir==NULL)||(oktext==NULL)||(canceltext==NULL)||(infotext==NULL)) return -1;
  if((actdir==NULL)&&(strlen(dir)==0)) return -1;
  reset();
  int w=200;
  int w2=strlen(oktext)+4+strlen(canceltext)+4;
  w2*=aguix->getCharWidth();
  w2+=30;
  if(w2>w) w=w2;
  w2=strlen(infotext)+2;
  w2*=aguix->getCharWidth();
  w2+=10;
  if(w2>w) w=w2;
  int h=3*aguix->getCharHeight()+50+150;

  AWindow *win=new AWindow(aguix);
  win->create(NULL,10,10,w,h,0,title);
  win->setMinSize(w,h);
  int ty,dh;
  ty=5;
  win->add(new Text(aguix,5,ty,infotext,1));
  ty+=aguix->getCharHeight()+5;
  lv=(ListView*)win->add(new ListView(aguix,5,ty,w-10,150,0));
  lv->setHBarState(2);
  lv->setVBarState(2);
  ty+=lv->getHeight();
  sg=(StringGadget*)win->add(new StringGadget(aguix,5,ty,w-10,dir,0));
//  sg->move(5,cnb->getY()-5-sg->getHeight());
//  dh-=sg->getHeight();
  ty+=sg->getHeight()+5;
//  int dh=h-dy;
  int dw=(strlen(oktext)+2)*aguix->getCharWidth();
  Button *okb=(Button*)win->add(new Button(aguix,5,ty,dw,oktext,1,0,0));
  dw=(strlen(canceltext)+2)*aguix->getCharWidth();
  Button *cnb=(Button*)win->add(new Button(aguix,w-5-dw,ty,dw,canceltext,1,0,0));
//  dh-=h-cnb->getY()+5;
  ty+=okb->getHeight()+5;
  h=ty;
  win->resize(w,h);
  win->setMinSize(w,h);
  win->show();
  AGMessage *msg;
  verz=new Verzeichnis();
  if(strlen(dir)>0) {
    setDir(dir);
  } else {
    setDir(actdir);
  }
  int endmode=-1;
  ListViewContent *lastlvc=NULL;
  Time lasttime=0;
  for(;endmode==-1;) {
    msg=aguix->WaitMessage(win);
    if(msg!=NULL) {
      switch(msg->type) {
        case AG_CLOSEWINDOW:
          if(msg->closewindow.window==win->getWindow()) endmode=0;
          break;
        case AG_BUTTONCLICKED:
          if(msg->button.button==okb) endmode=1;
          else if(msg->button.button==cnb) endmode=0;
          break;
        case AG_SIZECHANGED:
          okb->move(5,msg->size.newh-5-okb->getHeight());
          dw=cnb->getWidth();
          cnb->move(msg->size.neww-dw-5,msg->size.newh-5-okb->getHeight());
          sg->move(5,cnb->getY()-5-sg->getHeight());
          sg->resize(msg->size.neww-10,sg->getHeight());
          dh=sg->getY()-10-aguix->getCharHeight();
          lv->resize(msg->size.neww-10,dh);
          break;
        case AG_STRINGGADGET_DEACTIVATE:
          if(msg->stringgadget.sg==sg) {
            setDir(sg->getText());
          }
          break;
        case AG_LISTVIEW_ONESELECT:
          if(msg->listview.lv==lv) {
            if(lastlvc!=NULL) {
              if(msg->listview.lvc==lastlvc) {
                // eventl. Doppelklick
                if((msg->listview.time-lasttime)<350) {
                  int x=lv->getPos(lastlvc);
                  if(x>=0) {
                    ArrayList *files=verz->getFiles();
                    FileEntry *fe=(FileEntry*)files->getElementAt(x);
                    if(fe!=NULL) {
                      if(fe->isDir()==true) {
                        char *tstr=(char*)_allocsafe(strlen(actdir)+1+strlen(fe->name)+2);
                        sprintf(tstr,"%s/%s",actdir,fe->name);
                        setDir(tstr);
                        _freesafe(tstr);
                      } else {
                        endmode=2;
                      }
                    }
                  }
                }
                if(endmode==-1) {
                  lastlvc=NULL;
                  lasttime=0;
                }
              } else {
                lastlvc=msg->listview.lvc;
                lasttime=msg->listview.time;
              }
            } else {
              lastlvc=msg->listview.lvc;
              lasttime=msg->listview.time;
            }
          }
          break;
        case AG_LISTVIEW_MULTISELECT:
          if(msg->listview.lv==lv) {
            lastlvc=NULL;
            lasttime=0;
          }
          break;
      }
      aguix->ReplyMessage(msg);
    }
  }
  if(endmode==1) {
    // Ok
    selfes=new List();
    ArrayList *files=verz->getFiles();
    FileEntry *fe=(FileEntry*)files->getElementAt(1);
    ListViewContent *lvc=(ListViewContent*)lv->get(1);
    while(fe!=NULL) {
      if(lvc->getSelect()==true) {
        selfes->addElement(fe->duplicate());
      }
      fe=(FileEntry*)files->getNextElement();
      lvc=(ListViewContent*)lv->getNext();
    }
  } else if(endmode==2) {
    selfes=new List();
    ArrayList *files=verz->getFiles();
    FileEntry *fe=(FileEntry*)files->getElementAt(1);
    ListViewContent *lvc=(ListViewContent*)lv->get(1);
    while(fe!=NULL) {
      if(lvc==lastlvc) {
        selfes->addElement(fe->duplicate());
        break;
      }
      fe=(FileEntry*)files->getNextElement();
      lvc=(ListViewContent*)lv->getNext();
    }
    endmode=1;
  }
  win->close();
  delete win;
  delete verz;
  verz=NULL;
  return endmode;
}

void FileRequester::reset()
{
  if(selfes!=NULL) {
    FileEntry *fe=(FileEntry*)selfes->getFirstElement();
    while(fe!=NULL) {
      delete fe;
      fe=(FileEntry*)selfes->getNextElement();
    }
    delete selfes;
    selfes=NULL;
  }
}

void FileRequester::setDir(const char *dir)
{
  char *tstr=HandlePath(dir);
  verz->closeDir();
  if(verz->readDir(tstr)!=0) {
    if(actdir!=NULL) {
      if(verz->readDir(actdir)!=0) return;
    } else verz->readDir("/");
  }
  _freesafe(tstr);
  tstr=dupstring(verz->getDir());
  if(actdir!=NULL) _freesafe(actdir);
  actdir=tstr;
  sg->setText(actdir);
  verz->sort(0);
  ArrayList *files=verz->getFiles();
  FileEntry *fe=(FileEntry*)files->getFirstElement();
  lv->removeAll();
  while(fe!=NULL) {
    if(fe->isDir()==false) {
      lv->add(new ListViewContent(fe->name,1,2,1,2,0,1,0,1,0));
    } else {
      lv->add(new ListViewContent(fe->name,3,2,3,2,0,3,0,3,0));
    }
    fe=(FileEntry*)files->getNextElement();
  }
  lv->redraw();
}

FileEntry *FileRequester::getFirstFE()
{
  if(selfes==NULL) return NULL;
  return (FileEntry*)selfes->getFirstElement();
}

FileEntry *FileRequester::getNextFE()
{
  if(selfes==NULL) return NULL;
  return (FileEntry*)selfes->getNextElement();
}

